import urllib
from sqlalchemy import create_engine


class Connector:
    @staticmethod
    def get_mssql_trusted_connection(host: str, instance: str, database: str):
        connection_string = f"SERVER={host}\\{instance};DATABASE={database};Trusted_Connection=yes;"
        quoted = urllib.parse.quote_plus("DRIVER={ODBC DRIVER 17 for SQL SERVER};" + connection_string)
        engine = create_engine('mssql+pyodbc:////?odbc_connect={}'.format(quoted))
        return engine.connect().connection



import pandas as pd


def insert_to_mssql_db(column_string, cursor, data_list, location, total_count, total, values):
    value_list = " union ".join(['select {}'.format(value) for value in values])
    execute_query = (
        f"insert into {location} ({column_string}) {value_list}"
    )
    try:
        cursor.execute(execute_query, data_list)
        progress = f'Progress: {total_count} / {total}'
        print(progress)
    except Exception as e:
        print(execute_query)
        print(data_list)
        raise e


class Loader:
    @staticmethod
    def insert_to_mssql_table(cursor, df: pd.DataFrame, schema: str, table: str):
        column_list = df.columns.tolist()
        column_list = [f'[{column}]' for column in column_list]
        column_string = ", ".join(column_list)
        location = f"{schema}.[{table}]"
        total_count = 0
        values = []
        data_list = []
        data_count = 0
        row_values = []
        total = df.shape[0]
        for column in df.columns:
            str_column = df[column].apply(str)
            max_size = str_column.str.len().max()
            if max_size > 256:
                row_values.append('cast ( ? as varchar(max))')
            else:
                row_values.append('?')
        row_value_list = ", ".join(row_values)
        for row in df.itertuples(index=False, name=None):
            row_size = len(row)
            total_count += 1
            data_count += row_size
            values.append(row_value_list)

            data_list.extend(row)
            next_size = data_count + row_size
            if next_size >= 2000:
                insert_to_mssql_db(column_string, cursor, data_list, location, total_count, total, values)
                values = []
                data_list = []
                data_count = 0
            insert_to_mssql_db(column_string, cursor, data_list, location, total_count, total, values)



import hashlib
import unittest
import pandas as pd
import numpy as np
from datetime import datetime
from src.etl.dataframe.cleaner import clean_float, clean_date, clean_int, hash_str, Cleaner


class TestCleanFunctions(unittest.TestCase):

    def test_clean_float(self):
        self.assertEqual(clean_float('123.45'), 123.45)
        self.assertEqual(clean_float('$1,234.56'), 1234.56)
        self.assertIsNone(clean_float(None))

    def test_clean_date(self):
        self.assertEqual(clean_date('2021-01-01'), datetime(2021, 1, 1))
        self.assertEqual(clean_date('01/01/2021'), datetime(2021, 1, 1))
        self.assertIsNone(clean_date(None))
        self.assertIsNone(clean_date(np.nan))

    def test_clean_int(self):
        self.assertEqual(clean_int(123), 123)
        self.assertEqual(clean_int(123.0), 123)
        with self.assertRaises(ValueError):
            clean_int(123.45)
        self.assertIsNone(clean_int(None))
        self.assertIsNone(clean_int(np.nan))

    def test_hash_str(self):
        self.assertEqual(hash_str('test'), hashlib.sha1('test'.encode()).hexdigest())


class TestCleaner(unittest.TestCase):

    def setUp(self):
        self.df = pd.DataFrame({
            'MixedCase': ['value1', 'value2'],
            'with spaces': ['value3', 'value4'],
            'with$Special#Chars': ['value5', 'value6']
        })

    def test_column_names_to_snake_case(self):
        expected_columns = ['mixed_case', 'with_spaces', 'with_dollars_special_num_chars']
        Cleaner.column_names_to_snake_case(self.df)
        self.assertEqual(expected_columns, self.df.columns.tolist())

    def test_clean_numbers(self):
        df = pd.DataFrame({
            'ints': ['$1,000', '2,000', '3,000'],
            'floats': ['$1,000.55', '2,000.66', '3,000.33']
        })
        Cleaner.clean_numbers(df)
        self.assertEqual(df['ints'].dtype, 'int64')
        self.assertEqual(df['floats'].dtype, 'float64')

    def test_clean_dates(self):
        df = pd.DataFrame({
            'dates': ['2021-01-01', '01/02/2021']
        })
        Cleaner.clean_dates(df)
        self.assertEqual(df['dates'].dtype, 'datetime64[ns]')

    def test_clean_bools(self):
        df = pd.DataFrame({
            'bools': ['yes', 'no', 'true', 'false']
        })
        Cleaner.clean_bools(df)
        self.assertEqual(df['bools'].dtype, 'bool')

    def test_clean_all(self):
        df = pd.DataFrame({
            'numbers': ['$1,000', '2,000', '3,000', '%100'],
            'dates': ['2021-01-01', '01/02/2021', '2021-01-01', '01/02/2021'],
            'bools': ['yes', 'no', 'true', 'false']
        })
        Cleaner.clean_all(df)
        self.assertEqual(df['numbers'].dtype, 'int64')
        self.assertEqual(df['dates'].dtype, 'datetime64[ns]')
        self.assertEqual(df['bools'].dtype, 'bool')

    def test_generate_hash_column(self):
        df = pd.DataFrame({
            'col1': ['value1', 'value2'],
            'col2': ['value3', 'value4']
        })
        expected_hash = df['col1'].apply(str) + df['col2'].apply(str)
        expected_hash = expected_hash.apply(hash_str)
        expected_hash.name = 'hash'
        result_df = Cleaner.generate_hash_column(df, ['col1', 'col2'], 'hash')
        result_series = result_df['hash']
        pd.testing.assert_series_equal(result_series, expected_hash)

    def test_coalesce_columns(self):
        df = pd.DataFrame({
            'col1': [None, 'B'],
            'col2': ['A', 'B'],
            'col3': [None, 'C']
        })
        expected_df = pd.DataFrame({
            'col1': [None, 'B'],
            'col2': ['A', 'B'],
            'col3': [None, 'C'],
            'coalesced': ['A', 'B']
        })
        result_df = Cleaner.coalesce_columns(df, ['col1', 'col2'], 'coalesced')
        pd.testing.assert_frame_equal(result_df, expected_df)


if __name__ == '__main__':
    unittest.main()



import unittest
import pandas as pd
from src.etl.dataframe.analyzer import Analyzer


class TestAnalyzer(unittest.TestCase):

    def setUp(self):
        # Setup sample DataFrames for testing
        self.df_single_id = pd.DataFrame({
            'id': [1, 2, 3, 4, 5],
            'name': ['Eve', 'Bob', 'Charlie', 'David', 'Eve'],
        })

        self.df_id_pairs = pd.DataFrame({
            'first': [1, 1, 2, 2, 3],
            'second': [1, 1, 2, 2, 3],
            'other': [10, 20, 30, 40, 10]
        })

        self.df_no_id_pairs = pd.DataFrame({
            'first': [1, 1, 2, 2, 3],
            'second': [1, 1, 2, 2, 3]
        })

    def test_find_single_id_candidate_columns(self):
        # Test case with single ID candidate
        expected_candidates = ['id']
        actual_candidates = Analyzer.find_single_id_candidate_columns(self.df_single_id)
        self.assertEqual(expected_candidates, actual_candidates)

        # Test case with no single ID candidate
        expected_candidates = []
        actual_candidates = Analyzer.find_single_id_candidate_columns(self.df_no_id_pairs)
        self.assertEqual(expected_candidates, actual_candidates)

    def test_find_id_pair_candidates(self):
        # Test case with ID pair candidates
        expected_pairs = [('first', 'other'), ('second', 'other')]
        actual_pairs = Analyzer.find_id_pair_candidates(self.df_id_pairs)
        self.assertEqual(expected_pairs, actual_pairs)

        # Test case with no ID pair candidates
        expected_pairs = []
        actual_pairs = Analyzer.find_id_pair_candidates(self.df_no_id_pairs)
        self.assertEqual(expected_pairs, actual_pairs)


if __name__ == '__main__':
    unittest.main()
